"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _effects = require("redux-saga/effects");
var _uuid = require("uuid");
var _initializer = require("../../initializer");
var _util = require("../../helpers/util");
var _actions = require("../actions");
var _stub = require("../../service/stub");
var _viewstatus = _interopRequireDefault(require("../../helpers/viewstatus"));
var _localStorage = require("../../service/localStorage");
var _labels = require("../../labels");
var _env_vars = require("../../env_vars");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
//import base64 from "react-native-base64";

//need to be changed

function sendRequestSaga(_ref) {
  let {
    options = {},
    context = {},
    auth = {},
    callback = () => {}
  } = _ref;
  return function* () {
    const showNotification = (0, _initializer.getNotificationMethod)();
    const showWarning = (0, _initializer.getWarningMethod)();
    try {
      const {
        usecaseId,
        withLoader,
        drawUsecaseId = true,
        newTab = false,
        vsLoader,
        autoDownload = true,
        mainTab = true,
        tabId: targetTabId
      } = context;
      const {
        tdlId
      } = options;
      let {
        username,
        password
      } = auth;
      if (!username) {
        const user = yield (0, _effects.select)(state => state.general.currentUser);
        if (user) username = user.username;
      }
      if (!password) {
        password = yield (0, _effects.select)(state => state.general.currentPassword);
        if (password && password.currentPassword) password = password.currentPassword;
      }
      const tabs = yield (0, _effects.select)(state => state.tabs.tabs);
      const currentActiveTab = yield (0, _effects.select)(state => state.tabs.currentActiveTab);
      yield (0, _effects.put)((0, _actions.setIsLoading)(typeof withLoader === "boolean" ? withLoader : !usecaseId));
      const headers = {};
      let requestData = {
        pageNumber: 0,
        pageSize: Number(_env_vars.DEFAULT_FETCH),
        rowsNumber: Number(_env_vars.DEFAULT_FETCH),
        ...options
      };
      if (username && password) {
        headers.authorization = "Basic".concat((0, _util.base64Encode)(unescape(encodeURIComponent("\"".concat(username, "\":\"").concat(password, "\"")))));
      }
      let oldVS = null;
      if (usecaseId) {
        oldVS = yield (0, _effects.select)(state => state.viewStatus[usecaseId]);
        if (oldVS) {
          yield (0, _effects.put)((0, _actions.changeUsecaseIsLoading)(oldVS.usecaseId, typeof vsLoader === "boolean" ? vsLoader : false));
          requestData = {
            ...requestData,
            ...oldVS.getRequestBody(options)
          };
        }
      }
      if (tdlId === "export" || tdlId === "downloadFile" || context.responseType === "blob") {
        headers["Content-Type"] = "blob";
        headers["Accept"] = "application/json, text/plain, */*";
      } else if (context.headers) {
        headers = {
          ...headers,
          ...context.headers
        };
      }
      let response = null;
      try {
        response = yield (0, _effects.call)(_stub.sendRequest, requestData, headers);
      } catch (e) {
        if (!e.response || typeof e.response.data === "string") {
          showNotification({
            message: (0, _labels.getTranslation)("error"),
            description: (0, _labels.getTranslation)("networkError"),
            type: "error"
          });
          yield (0, _effects.put)((0, _actions.setIsLoading)(false));
          if (oldVS) {
            yield (0, _effects.put)((0, _actions.changeUsecaseIsLoading)(oldVS.usecaseId, false));
          }
          if (callback) yield (0, _effects.call)(callback, null);
          return;
        }
        response = e.response;
        console.log({
          e
        });
      }
      if (context.responseType === "free") {
        if (callback) yield (0, _effects.call)(callback, response);
        return;
      } else if (headers["Content-Type"] === "blob") {
        yield (0, _effects.all)([(0, _effects.put)((0, _actions.setIsLoading)(false)), (0, _effects.put)((0, _actions.changeUsecaseIsLoading)(oldVS.usecaseId, false))]);
        if (autoDownload) {
          if ((0, _initializer.getDownloadMethod)()) {
            (0, _initializer.getDownloadMethod)()(response);
          }
        }
        if (callback) yield (0, _effects.call)(callback, response);
        return;
      }
      const {
        isSessionInvalidated,
        isRTL,
        timeZoneOffset,
        userName: responseUsername,
        agency,
        data,
        output
      } = response.data;
      if (!context.disableMessages) {
        const {
          errorMessages,
          successMessages,
          warningMessages
        } = output;
        errorMessages.map(item => {
          console.log("omg");
          showNotification({
            message: item.title,
            description: item.message || (0, _labels.getTranslation)("error".concat(response.status)),
            type: "error"
          });
        });
        successMessages.map(item => showNotification({
          message: item.title,
          description: item.message,
          type: "success"
        }));
        warningMessages.map(item => {
          showWarning(item, requestData.tdlId, context.usecaseId);
        });
      }
      if (response.status === 500) {
        yield (0, _effects.put)((0, _actions.setIsLoading)(false));
        return;
      }
      if (options.tdlId === "CMS$$CMS$$-$$favouriteMenu") {
        const favoriteItems = data.diffObject;
        yield (0, _effects.put)((0, _actions.setFavoriteItems)(favoriteItems.filter(item => item.newTdlId && !item.newTdlId.startsWith("%"))));
        const currentUser = yield (0, _effects.select)(state => state.general.currentUser);
        (0, _localStorage.setFavoriteItems)(currentUser, favoriteItems);
      } else if (options.tdlId === "EDMS$$modeling$$-$$mainmenue") {
        const mainMenuItems = (0, _util.transformMainMenu)(data.diffObject);
        yield (0, _effects.put)((0, _actions.setMainMenu)(mainMenuItems));
        const currentUser = yield (0, _effects.select)(state => state.general.currentUser);
        (0, _localStorage.setMainMenu)(currentUser, mainMenuItems);
      }
      let vs = null;
      let currentTab = null;
      let backAction = false;
      let changeCurrent = false;
      let usecaseIdToClose = null;
      if (data) {
        let newVS = null;
        if (data && data.usecaseId) {
          newVS = yield (0, _effects.select)(state => state.viewStatus[data.usecaseId]);
        }

        // We will generate a Random ID for the tab in case we needed it , it can change below
        let tabId = targetTabId || (0, _uuid.v4)();

        // If we don't want to draw new tab anyway
        if (!newTab && !targetTabId) {
          // If we have a newVS that means that we have current tab for the VS already , so tabId = vs tabI
          if (newVS) {
            tabId = newVS.tabId;
            changeCurrent = true;
            // If we have an oldVs we will use the same tabId
          } else if (oldVS) {
            tabId = oldVS.tabId;
            changeCurrent = true;
            // If we don't have a newVs and we don't have oldVs but we want to render it as mainTab so check if there is a current active tab to set it to
          } else if (mainTab) {
            currentTab = tabs.find(item => item.id === currentActiveTab);
            if (currentTab) {
              tabId = currentActiveTab;
              changeCurrent = true;
            }
          }
        }

        // Is no Old view status then it's new tdlId
        if (!oldVS) {
          vs = new _viewstatus.default(requestData, response.data);
          vs.tabId = tabId;
          usecaseIdToClose = currentTab && currentTab.usecaseId;
        }
        // Id there is oldVS and no newVS then it's forward
        else if (!newVS) {
          vs = new _viewstatus.default(requestData, response.data);
          vs.tabId = tabId;
          vs.callerUsecaseId = oldVS.usecaseId;
          vs.callersLabelsArray = [...oldVS.callersLabelsArray, {
            tdlLabel: oldVS.tdlLabel,
            usecaseId: oldVS.usecaseId
          }];
        }
        // If oldVS and newVS are the same
        else if (oldVS.usecaseId === newVS.usecaseId) {
          vs = (0, _util.cloneDeep)(newVS);
          vs.dealWithSuccess(requestData, response.data);
        }
        // If oldVS and newVS then and there are not the same then it's back
        else {
          vs = (0, _util.cloneDeep)(newVS);
          vs.dealWithSuccess(requestData, response.data);
          backAction = true;
          if (oldVS.isPopUp()) {
            yield (0, _effects.put)((0, _actions.closeTab)("popup"));
          }
        }
      }
      if (tdlId === "closeUseCase" || context.closeUsecase) {
        if (oldVS) {
          usecaseIdToClose = context.closeUsecase ? oldVS.usecaseId : null;
          yield (0, _effects.put)((0, _actions.removeUsecase)(oldVS.usecaseId));
          oldVS = null;
        }
        if (vs) {
          usecaseIdToClose = context.closeUsecase ? vs.usecaseId : null;
          yield (0, _effects.put)((0, _actions.removeUsecase)(vs.usecaseId));
          vs = null;
        }
      }
      const isPopup = vs && vs.isPopUp();
      if (isPopup) {
        vs.tabId = "popup";
      }
      yield (0, _effects.all)([(0, _effects.put)((0, _actions.setCurrentUser)({
        username: responseUsername,
        agency
      })), (0, _effects.put)((0, _actions.setIsSessionInvalidated)(isSessionInvalidated)), (0, _effects.put)((0, _actions.setIsRTL)(isRTL)), (0, _effects.put)((0, _actions.setTimezoneOffset)(timeZoneOffset)), vs && (0, _effects.put)((0, _actions.updateViewStatus)(vs)), oldVS && (0, _effects.put)((0, _actions.changeUsecaseIsLoading)(oldVS.usecaseId, false)), isSessionInvalidated && (0, _effects.put)((0, _actions.closeAllTabs)()), isSessionInvalidated && (0, _effects.put)((0, _actions.resetViewStatuses)()), backAction && (0, _effects.put)((0, _actions.removeUsecase)(oldVS.usecaseId)), vs && drawUsecaseId && !changeCurrent && (0, _effects.put)((0, _actions.createNewTab)(vs.usecaseId, vs.tabId, isPopup ? false : mainTab)), changeCurrent && drawUsecaseId && vs && (0, _effects.put)((0, _actions.changeTabVS)(vs.usecaseId, vs.tabId)), usecaseIdToClose && (0, _effects.put)((0, _actions.closeUsecaseId)(usecaseIdToClose)), callback && (0, _effects.call)(callback, response, vs), (0, _effects.put)((0, _actions.setIsLoading)(false))]);
    } catch (e) {
      try {
        console.log(e);
      } catch (e2) {
        console.log("something went worng in saga");
      }
    }
  }();
}
function uploadFileSaga(_ref2) {
  let {
    usecaseId,
    file,
    path,
    callback = () => {}
  } = _ref2;
  return function* () {
    yield (0, _effects.put)((0, _actions.changeUsecaseIsLoading)(usecaseId, true));
    const formData = new FormData();
    formData.append("upload", file);
    formData.append("useCaseId", usecaseId);
    formData.append("fieldPath", path);
    const params = {
      fieldPath: path,
      useCaseId: usecaseId
    };
    let response = null;
    try {
      response = yield (0, _effects.call)(_stub.uploadFile, formData, params);
      if (callback) yield (0, _effects.call)(callback, response);
      yield (0, _effects.put)((0, _actions.sendRefreshWithoutUpdate)(usecaseId));
    } catch (e) {
      showNotification({
        message: (0, _labels.getTranslation)("errorUpload"),
        description: e.message
      });
    } finally {
      yield (0, _effects.put)((0, _actions.changeUsecaseIsLoading)(usecaseId, true));
    }
  }();
}
var _default = exports.default = [(0, _effects.takeEvery)(_actions.GENERAL_ACTION_TYPES.SEND_REQUEST, sendRequestSaga), (0, _effects.takeEvery)(_actions.GENERAL_ACTION_TYPES.UPLOAD_FILE, uploadFileSaga)];